#include "binding.h"

class EventListener : public gs_InputListener
{
public:
	EventListener( gs_InputManager *manager, gs_Binding* parent, int checkflags)
	{ 
		Device = 0;
		Manager = manager;
		Parent = parent; 
		CheckFlags = checkflags;
	}

	EventListener( gs_InputDevice *device, gs_Binding* parent, int checkflags)
	{ 
		Device = device;
		Manager = 0;
		Parent = parent; 
		CheckFlags = checkflags;
	}


	int Message( gs_InputState* s) 
	{
		// check if this is the type of input wanted
		int InputType = 0;

		if (s->Index < s->Parent->GetTotalAxes())
			InputType = AXISONLY;
		else if (s->Index < s->Parent->GetTotalPOVs())
			InputType = POVONLY;
		else if (s->Index < s->Parent->GetTotalButtons())
			InputType = BUTTONONLY;

		if (CheckFlags == InputType)
		{
			Parent->Connect(&s->Parent->GetInput(), s->Index);

			// clean up
			if (Manager)
			{
				// added notify to multiple devices
				for(int x = 0; x < Manager->GetTotal(); x++)
					Manager->Get(x)->RemoveNotify(this);
			}
			else	// just clean up single device
				Device->RemoveNotify(this);		
		

			delete this;	// I know this is not right, but it is the best solution I could think of to get rid of object after connection
			return 1;
		}
		return 1;
	}

	gs_InputDevice *Device;
	gs_InputManager *Manager;
	gs_Binding* Parent;
	int CheckFlags;
};

int gs_Binding::ConnectFirst(gs_InputManager* manager, int flags)
{
	Disconnect();

	EventListener* e = new EventListener( manager, this, flags);

	for( int x = 0; x < manager->GetTotal(); x++)
		manager->Get(x)->AddNotify(e);
	return 1;
}

int gs_Binding::ConnectFirst(gs_InputDevice* device, int flags)
{
	Disconnect();
	device->AddNotify( new EventListener( device, this, flags));

	return 1;
}

void gs_Binding::Connect( gs_InputData* source, int index)
{
	Source = source;
	Index = index;
}

int gs_Binding::Get()
{
	if (!Source)
		return 0;
	return Source->Get(Index);
}
